var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Application_applicationId, _Application_publicKey, _Application_token, _Application_commands, _Application_components, _Application_shutdown;
import debug from "debug";
import { client } from "discord-api-methods";
import { ApplicationCommandType } from "discord-api-types/v10";
import { isValidRequest } from "discord-verify";
import { isExecutableComponent } from "./components/index.js";
import Config from "./config.js";
import * as Interaction from "./interactions/index.js";
import { ResponseStatus, response } from "./requests/response.js";
const log = debug("ikit:application");
export default class Application {
    constructor({ applicationId, publicKey, token }) {
        _Application_applicationId.set(this, void 0);
        _Application_publicKey.set(this, void 0);
        _Application_token.set(this, void 0);
        _Application_commands.set(this, void 0);
        _Application_components.set(this, new Map());
        _Application_shutdown.set(this, void 0);
        if (!applicationId) {
            throw new Error("Please provide an Application ID. You can find this value <here>");
        }
        if (!publicKey) {
            throw new Error("Please provide a Public Key. You can find this value <here>");
        }
        if (!token) {
            throw new Error("Please provide a Token. You can find this value <here>");
        }
        __classPrivateFieldSet(this, _Application_applicationId, applicationId, "f");
        __classPrivateFieldSet(this, _Application_publicKey, publicKey, "f");
        __classPrivateFieldSet(this, _Application_token, token, "f");
        __classPrivateFieldSet(this, _Application_shutdown, new AbortController(), "f");
        // Set up internal data structures
        __classPrivateFieldSet(this, _Application_commands, {
            [ApplicationCommandType.ChatInput]: new Map(),
            [ApplicationCommandType.Message]: new Map(),
            [ApplicationCommandType.User]: new Map(),
        }, "f");
        // Configure API Defaults
        Config.setApplicationId(__classPrivateFieldGet(this, _Application_applicationId, "f"));
        client.setToken(__classPrivateFieldGet(this, _Application_token, "f"));
        client.abortSignal = __classPrivateFieldGet(this, _Application_shutdown, "f").signal;
    }
    get id() {
        return __classPrivateFieldGet(this, _Application_applicationId, "f");
    }
    get commands() {
        return Object.values(__classPrivateFieldGet(this, _Application_commands, "f"))
            .map((map) => [...map.values()])
            .flat();
    }
    addCommand(command) {
        if (__classPrivateFieldGet(this, _Application_commands, "f")[command.type]?.has(command.name.toLowerCase())) {
            throw new Error(`Error registering ${command.name.toLowerCase()}: Duplicate names are not allowed`);
        }
        log(`Adding the ${command.name.toLowerCase()} command`);
        __classPrivateFieldGet(this, _Application_commands, "f")[command.type].set(command.name.toLowerCase(), 
        // TypeScript constraint due to command being all possible command types, it's difficult to type tersely
        // https://canary.discord.com/channels/508357248330760243/746364189710483546/900780684690485300
        command);
        return this;
    }
    addCommands(...commands) {
        commands.forEach((command) => this.addCommand(command));
        return this;
    }
    getCommand(type, name) {
        // I'm not sure why, but this needs to be cast to prevent an error
        return __classPrivateFieldGet(this, _Application_commands, "f")[type].get(name);
    }
    addComponent(component) {
        if (isExecutableComponent(component) &&
            component.id != null &&
            !__classPrivateFieldGet(this, _Application_components, "f").has(component.id)) {
            log(`Registering the ${component.id} component`);
            __classPrivateFieldGet(this, _Application_components, "f").set(component.id, component);
        }
        return this;
    }
    getComponent(customId) {
        return __classPrivateFieldGet(this, _Application_components, "f").get(customId);
    }
    async findComponent(customId) {
        for (const component of __classPrivateFieldGet(this, _Application_components, "f").values()) {
            const match = await component.matches?.(customId);
            if (match) {
                return component;
            }
        }
        return undefined;
    }
    // TODO: This is convoluted and doesn't really work. We need to return
    // two states; one promise should resolve to the current response,
    // and the second should resolve to "all the work is done, we can end the process".
    // This will be important for serverless and worker environments
    async handler(request) {
        log("Handling Incoming Request");
        if (request.method !== "POST") {
            return response(ResponseStatus.MethodNotAllowed, {
                error: "Invalid Method",
            });
        }
        // TODO: Handle different platforms
        const valid = await isValidRequest(request, __classPrivateFieldGet(this, _Application_publicKey, "f"));
        if (!valid) {
            return response(ResponseStatus.Unauthorized, {
                error: "Invalid Request",
            });
        }
        try {
            const json = await request.json();
            return await new Promise((resolve) => {
                void Interaction.handler(this, json, (status, json) => {
                    resolve(response(status, json));
                }, request);
            });
        }
        catch (error) {
            log(error.message);
            return response(ResponseStatus.BadRequest, { error: "Unknown Type" });
        }
    }
    shutdown() {
        __classPrivateFieldGet(this, _Application_shutdown, "f").abort();
    }
}
_Application_applicationId = new WeakMap(), _Application_publicKey = new WeakMap(), _Application_token = new WeakMap(), _Application_commands = new WeakMap(), _Application_components = new WeakMap(), _Application_shutdown = new WeakMap();
//# sourceMappingURL=application.js.map