import { ApplicationCommandOptionType } from "discord-api-types/v10";
import { BasicOption } from "./option.js";
export default class StringOption extends BasicOption {
    constructor({ choices, autocomplete, name, description, required, }) {
        super({
            type: ApplicationCommandOptionType.String,
            name,
            description,
            required,
        });
        /* eslint-disable-next-line no-negated-condition */
        if (autocomplete != null) {
            this.autocomplete = autocomplete;
        }
        else {
            this.choices = choices;
        }
    }
    isAutocomplete(_payload) {
        return this.autocomplete != null;
    }
    equals(schema) {
        if (schema.autocomplete) {
            if (this.autocomplete == null) {
                return false;
            }
            if (this.choices?._choices.size !== 0) {
                return false;
            }
        }
        else {
            if (this.autocomplete != null) {
                return false;
            }
            if ((this.choices?._choices?.size ?? 0) !== (schema.choices?.length ?? 0)) {
                return false;
            }
            const choiceMap = new Map();
            for (const commandOption of this.choices?._choices?.values() ?? []) {
                choiceMap.set(commandOption.name, commandOption.value);
            }
            const choiceEquality = schema.choices?.every((choice) => {
                if (!choiceMap.has(choice.name)) {
                    return false;
                }
                return choiceMap.get(choice.name) === choice.value;
            }) ?? true;
            if (!choiceEquality) {
                return false;
            }
        }
        return super.equals(schema);
    }
    serialize() {
        const payload = super.serialize();
        if (this.isAutocomplete(payload)) {
            payload.autocomplete = true;
        }
        else if (this.choices != null) {
            payload.choices = this.choices.serialize();
        }
        return payload;
    }
}
//# sourceMappingURL=string.js.map