import { ApplicationCommandOptionType } from "discord-api-types/v10";
import { Option } from "./option.js";
export default class SubcommandGroup extends Option {
    constructor({ name, description, subcommands }) {
        super({
            type: ApplicationCommandOptionType.SubcommandGroup,
            name,
            description,
        });
        this.type = ApplicationCommandOptionType.SubcommandGroup;
        this.subcommands = new Map(subcommands.map((cmd) => [cmd.name, cmd]));
    }
    serialize() {
        const payload = super.serialize();
        payload.options = [...this.subcommands.values()].map((subcommand) => subcommand.serialize());
        return payload;
    }
    equals(schema) {
        if (this.subcommands.size !== schema.options?.length) {
            return false;
        }
        const serializedSubcommands = [...this.subcommands.values()].map((subcommand) => subcommand.serialize());
        if (serializedSubcommands !== schema.options) {
            return false;
        }
        return super.equals(schema);
    }
}
//# sourceMappingURL=subcommand-group.js.map