import { ApplicationCommandOptionType } from "discord-api-types/v10";
import { Option } from "./option.js";
export default class Subcommand extends Option {
    constructor({ name, description, options, handler, autocomplete, }) {
        super({
            type: ApplicationCommandOptionType.Subcommand,
            name,
            description,
        });
        this.type = ApplicationCommandOptionType.Subcommand;
        // TODO: This is somewhat shared with SlashCommand.
        // TODO: Validate: 1-32 lowercase character name matching ^[\w-]{1,32}$
        this.options = new Map();
        this.handler = handler;
        if (autocomplete != null)
            this.autocomplete = autocomplete;
        options?.forEach((option) => {
            const key = option.name.toLowerCase();
            if (this.options.has(key)) {
                throw new Error(`Option names must be unique (case insensitive). Duplicate name detected: ${key}`);
            }
            this.options.set(key, option);
        });
    }
    equals(schema) {
        if (!super.equals(schema)) {
            return false;
        }
        if (this.options.size !== (schema.options?.length ?? 0)) {
            return false;
        }
        return (schema.options?.every((option) => this.options.get(option.name)?.equals(option) ?? false) ?? true);
    }
    serialize() {
        const payload = super.serialize();
        // TODO: Sort these so that required options come first
        if (this.options.size > 0) {
            payload.options = Array.from(this.options.values()).map((option) => option.serialize());
        }
        return payload;
    }
}
//# sourceMappingURL=subcommand.js.map