import type { APIApplicationCommand, APIChatInputApplicationCommandInteraction, PermissionFlagsBits, RESTPostAPIChatInputApplicationCommandsJSONBody } from "discord-api-types/v10";
import { ApplicationCommandType } from "discord-api-types/v10";
import type SlashCommandInteraction from "../interactions/application-commands/slash-command-interaction.js";
import type SlashCommandAutocompleteInteraction from "../interactions/autocomplete/application-command-autocomplete.js";
import type { Autocomplete } from "../interactions/autocomplete/types.js";
import type { ArrayValue, InteractionKitCommand, Optional, RequestBody, ValueOf } from "../interfaces.js";
import { SubcommandGroup } from "./options/index.js";
import type { BasicOption } from "./options/option.js";
import Subcommand from "./options/subcommand.js";
export declare type CommandArgs = {
    name: string;
    description: string;
    defaultPermissions?: Array<ValueOf<typeof PermissionFlagsBits>>;
    options?: BasicOption[];
    handler: InteractionKitCommand<SlashCommandInteraction>["handler"];
    autocomplete?: never;
    commands?: never;
};
export declare type ParentCommandArgs = {
    name: string;
    description: string;
    defaultPermissions?: never;
    options?: never;
    handler?: never;
    autocomplete?: never;
    commands: Array<Subcommand | SubcommandGroup>;
};
export declare type AutocompleteCommandArgs = CommandArgs & {
    options?: never;
    autocomplete?: Autocomplete<SlashCommandAutocompleteInteraction>["autocomplete"];
};
export default class SlashCommand implements Optional<InteractionKitCommand<SlashCommandInteraction>, "handler">, Autocomplete<SlashCommandAutocompleteInteraction> {
    #private;
    readonly type = ApplicationCommandType.ChatInput;
    name: string;
    options: Map<string, BasicOption>;
    defaultPermissions: null | bigint;
    handler?: InteractionKitCommand<SlashCommandInteraction>["handler"];
    autocomplete?: Autocomplete<SlashCommandAutocompleteInteraction>["autocomplete"];
    commands: Map<string, ArrayValue<ParentCommandArgs["commands"]>>;
    constructor({ name, description, options, defaultPermissions, handler, autocomplete, commands, }: CommandArgs | ParentCommandArgs | AutocompleteCommandArgs);
    equals(schema: APIApplicationCommand): boolean;
    getCommandHandler(json: RequestBody<APIChatInputApplicationCommandInteraction>): (interaction: SlashCommandInteraction, application: import("../application.js").default, request: Request) => import("../interfaces.js").Awaitable<void>;
    serialize(): RESTPostAPIChatInputApplicationCommandsJSONBody;
}
