var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _SlashCommand_description;
import { ApplicationCommandOptionType, ApplicationCommandType, } from "discord-api-types/v10";
import { SubcommandGroup } from "./options/index.js";
import Subcommand from "./options/subcommand.js";
export default class SlashCommand {
    constructor({ name, description, options, defaultPermissions, handler, autocomplete, commands, }) {
        this.type = ApplicationCommandType.ChatInput;
        _SlashCommand_description.set(this, void 0);
        // TODO: Validate: 1-32 lowercase character name matching ^[\w-]{1,32}$
        this.name = name;
        __classPrivateFieldSet(this, _SlashCommand_description, description, "f");
        this.options = new Map();
        this.commands = new Map(commands?.map((cmd) => [cmd.name, cmd]));
        if (handler != null) {
            this.handler = handler;
        }
        if (autocomplete != null) {
            this.autocomplete = autocomplete;
        }
        options?.forEach((option) => {
            const key = option.name.toLowerCase();
            if (this.options.has(key)) {
                throw new Error(`Option names must be unique (case insensitive). Duplicate name detected: ${key}`);
            }
            this.options.set(key, option);
        });
        this.defaultPermissions =
            defaultPermissions?.reduce((value, permission) => value | permission, 0n) ?? null;
    }
    equals(schema) {
        if (this.name !== schema.name || __classPrivateFieldGet(this, _SlashCommand_description, "f") !== schema.description) {
            return false;
        }
        if (this.options.size !== (schema.options?.length ?? 0)) {
            return false;
        }
        if (this.defaultPermissions !== schema.default_member_permissions) {
            return false;
        }
        if (this.commands.size > 0) {
            // This is a parent command
            return (schema.options?.every((option) => {
                const subcommand = this.commands.get(option.name);
                if (subcommand == null) {
                    return false;
                }
                if (subcommand.type === ApplicationCommandOptionType.Subcommand) {
                    return subcommand.equals(option);
                }
                return subcommand.equals(option);
            }) ?? true);
        }
        // This is a normal slash command
        return (schema.options?.every((option) => this.options
            .get(option.name)
            ?.equals(option) ?? false) ?? true);
    }
    getCommandHandler(json) {
        let options = this.commands;
        let interactionOptions = json.data.options;
        if (interactionOptions?.[0]?.type ===
            ApplicationCommandOptionType.SubcommandGroup) {
            const option = options.get(interactionOptions[0].name);
            interactionOptions = interactionOptions[0].options;
            if (option instanceof SubcommandGroup) {
                options = option.subcommands;
            }
        }
        if (interactionOptions?.[0]?.type === ApplicationCommandOptionType.Subcommand) {
            const option = options.get(interactionOptions[0].name);
            if (option instanceof Subcommand) {
                return option.handler;
            }
        }
        if (this.handler == null) {
            // TODO: Throw actual error
            throw new Error(`Unknown handler for interaction ${json.data.name}`);
        }
        return this.handler;
    }
    serialize() {
        const payload = {
            name: this.name,
            description: __classPrivateFieldGet(this, _SlashCommand_description, "f"),
        };
        // TODO: Sort these so that required options come first
        if (this.options.size > 0) {
            payload.options = [];
            Array.from(this.options.entries()).forEach(([_, value]) => {
                payload.options?.push(value.serialize());
            });
        }
        if (this.defaultPermissions != null) {
            payload.default_member_permissions = this.defaultPermissions.toString();
        }
        if (this.commands.size > 0) {
            payload.options = Array.from(this.commands.values()).map((cmd) => cmd.serialize());
        }
        return payload;
    }
}
_SlashCommand_description = new WeakMap();
//# sourceMappingURL=slash-command.js.map