import type { Executable, SerializableComponent } from "../interfaces.js";
import type { APIBaseComponent, APIButtonComponent, APIButtonComponentWithCustomId, APIButtonComponentWithURL, APIMessageComponentEmoji } from "discord-api-types/v10";
import { ButtonStyle, ComponentType } from "discord-api-types/v10";
import type ButtonInteraction from "../interactions/message-components/button-interaction.js";
interface APIButtonComponentBase<Style extends ButtonStyle> extends APIBaseComponent<ComponentType.Button> {
    label?: string;
    style: Style;
    emoji?: APIMessageComponentEmoji;
    disabled?: boolean;
}
declare type ButtonBaseArgs<T extends ButtonStyle> = Omit<APIButtonComponent, "type" | "url" | "custom_id" | "components" | "options" | "placeholder" | "min_values" | "max_values" | "style" | "label" | "emoji" | "disabled"> & {
    style: T;
    label?: APIButtonComponent["label"] | undefined;
    emoji?: APIButtonComponent["emoji"] | undefined;
    disabled?: APIButtonComponent["disabled"] | undefined;
};
declare type ButtonArgs = {
    customId: APIButtonComponentWithCustomId["custom_id"];
    handler: Executable<ButtonInteraction>["handler"];
    style: Exclude<ButtonStyle, ButtonStyle.Link>;
    matches?: Executable<ButtonInteraction>["matches"];
} & ButtonBaseArgs<Exclude<ButtonStyle, ButtonStyle.Link>>;
declare type ButtonLinkArgs = Omit<APIButtonComponentWithURL, "type" | "custom_id" | "style" | "components">;
declare abstract class ButtonBase<T extends ButtonStyle> {
    #private;
    constructor({ label, emoji, disabled, style }: ButtonBaseArgs<T>);
    get id(): SerializableComponent["id"];
    get type(): ComponentType;
    setLabel(label: ButtonArgs["label"]): this;
    setEmoji(emoji: ButtonArgs["emoji"]): this;
    setDisabled(disabled: ButtonArgs["disabled"]): this;
    serialize(): APIButtonComponentBase<T>;
    protected setStyle(style: T): this;
}
export declare class ButtonLink extends ButtonBase<ButtonStyle.Link> {
    #private;
    constructor({ disabled, emoji, label, url }: ButtonLinkArgs);
    setURL(url: ButtonLinkArgs["url"]): this;
    serialize(): APIButtonComponentWithURL;
}
export declare class Button extends ButtonBase<Exclude<ButtonStyle, ButtonStyle.Link>> implements SerializableComponent, Executable<ButtonInteraction> {
    #private;
    matches: ButtonArgs["matches"];
    handler: ButtonArgs["handler"];
    constructor({ handler, customId, style, matches, disabled, emoji, label, }: ButtonArgs);
    get id(): string;
    setCustomId(customId: ButtonArgs["customId"]): this;
    setStyle(style: Exclude<ButtonArgs["style"], ButtonStyle.Link>): this;
    setHandler(fn: ButtonArgs["handler"]): this;
    setMatches(fn: ButtonArgs["matches"]): this;
    serialize(): APIButtonComponentWithCustomId;
}
export {};
