import type { APIEmbed, APIEmbedAuthor, APIEmbedField, APIEmbedFooter, APIEmbedImage, APIEmbedThumbnail } from "discord-api-types/v10";
declare type FooterOptions = Omit<APIEmbedFooter, "icon_url" | "proxy_icon_url"> & {
    iconURL: APIEmbedFooter["icon_url"];
};
declare type AuthorOptions = Omit<APIEmbedAuthor, "icon_url" | "proxy_icon_url"> & {
    iconURL: APIEmbedAuthor["icon_url"];
};
declare type EmbedArgs = Omit<APIEmbed, "footer" | "author"> & {
    footer?: FooterOptions;
    author?: AuthorOptions;
};
export default class Embed {
    title: string | undefined;
    description: string | undefined;
    url: string | undefined;
    timestamp: string | undefined;
    color: number | undefined;
    footer: APIEmbedFooter | undefined;
    image: APIEmbedImage | undefined;
    thumbnail: APIEmbedThumbnail | undefined;
    author: APIEmbedAuthor | undefined;
    fields: APIEmbedField[] | undefined;
    constructor({ title, description, url, timestamp, color, footer, image, thumbnail, author, fields, }?: EmbedArgs);
    setTitle(title: string): this;
    setDescription(description: string): this;
    setURL(url: string): this;
    setTimestamp(timestamp?: number | Date): this;
    setColor(color: number): this;
    setFooter({ text, iconURL }: FooterOptions): this;
    setImage(url: string): this;
    setThumbnail(url: string): this;
    setAuthor({ name, url, iconURL }: AuthorOptions): this;
    setFields(fields: APIEmbedField[]): this;
    addField(field: APIEmbedField[]): this;
    serialize(): APIEmbed;
}
export {};
