export default class Embed {
    constructor({ title, description, url, timestamp, color, footer, image, thumbnail, author, fields, } = {}) {
        this.title = title;
        this.description = description;
        this.url = url;
        this.color = color;
        this.footer = footer;
        this.image = image;
        this.thumbnail = thumbnail;
        this.author = author;
        this.fields = fields;
        if (timestamp != null) {
            this.timestamp = new Date(timestamp).toISOString();
        }
    }
    setTitle(title) {
        this.title = title;
        return this;
    }
    setDescription(description) {
        this.description = description;
        return this;
    }
    setURL(url) {
        this.url = url;
        return this;
    }
    setTimestamp(timestamp = Date.now()) {
        this.timestamp = new Date(timestamp).toISOString();
        return this;
    }
    setColor(color) {
        this.color = color;
        return this;
    }
    setFooter({ text, iconURL }) {
        const footer = { text };
        if (iconURL != null) {
            footer.icon_url = iconURL;
        }
        this.footer = footer;
        return this;
    }
    setImage(url) {
        this.image = {
            url,
        };
        return this;
    }
    setThumbnail(url) {
        this.thumbnail = {
            url,
        };
        return this;
    }
    setAuthor({ name, url, iconURL }) {
        const author = { name };
        if (url != null) {
            author.url = url;
        }
        if (iconURL != null) {
            author.icon_url = iconURL;
        }
        this.author = author;
        return this;
    }
    setFields(fields) {
        this.fields = fields;
        return this;
    }
    addField(field) {
        return this.setFields((this.fields ?? []).concat(field));
    }
    serialize() {
        // @ts-expect-error it really wants me to do this as optional args
        return {
            title: this.title,
            description: this.description,
            url: this.url,
            timestamp: this.timestamp,
            color: this.color,
            footer: this.footer,
            image: this.image,
            thumbnail: this.thumbnail,
            author: this.author,
            fields: this.fields,
        };
    }
}
//# sourceMappingURL=embed.js.map